# -*- coding: utf-8 -*-

import unittest
import os
import codecs
import logging
import cgi

from time import time
from datetime import timedelta

from lucene import *

from analysis.koreananalyzer import KoreanAnalyzer

class SearcherService(object):
    def __init__(self, index_dir, database_getter=None):
        self.index_dir = index_dir
        self.kanal = KoreanAnalyzer()
        self.queryparser = QueryParser("content", self.kanal)
        self.queryparser.setDefaultOperator(QueryParser.Operator.AND)
        self.load()
        self.logger = logging.getLogger("search")
        self.database_getter = database_getter

    def reload(self):
        # Make a new indexsearcher to refresh
        # XXX: Not THREAD SAFE (Hits object and things might be lost!)
        fsDir = RAMDirectory(FSDirectory.getDirectory(self.index_dir, False))
        new_searcher = IndexSearcher(fsDir)
        old_searcher = self.searcher
        self.searcher = new_searcher
        old_searcher.close()
        self.last_load_time = time()

    def load(self):
        fsDir = RAMDirectory(FSDirectory.getDirectory(self.index_dir, False))
        self.searcher = IndexSearcher(fsDir)
        self.last_load_time = time()


    def search(self, q, from_=0, to_=10, source="web", sort="score"):
        start = time()
        DEBUG = False
        if not self.searcher.getIndexReader().isCurrent():
            self.reload()
        if "debug:true" in q:
            DEBUG = True
            q = q.replace("debug:true", "")
        filter_ = TermsFilter()
        filter_.addTerm(Term("source", source))
        sort = Sort()
        if sort == "date":
            sort = Sort("date")
        if sort == "-date":
            sort = Sort("date", True)
        try:
            query = self.queryparser.parse(q)
        except JavaError:
            self.logger.info("Malformed query: %s", q)
            result = {'results':0, 'hits':[]}
            result['search_time'] = 0.0
            result['search_whole_time'] = 0.0
            return result

        scorer = QueryScorer(query)
        formatter = SimpleHTMLFormatter("__LT__", "__GT__")
        highlighter = Highlighter(formatter, scorer)
        analyzer = KoreanAnalyzer()

        hits = self.searcher.search(query, filter_, sort)
        search_duration = time() - start

        result = {'results':hits.length(), 'hits':[]}

        if DEBUG:
            result['query'] = query.toString()

        for i in range(from_, min(to_, hits.length())):
            item = {}
            doc = hits.doc(i)
            content = doc.get("body")[:1024*1024*1]
            stream = TokenSources.getAnyTokenStream(self.searcher.getIndexReader(), hits.id(i), "body", analyzer);
            fragment = highlighter.getBestFragment(stream, content)
            fragment = cgi.escape(fragment or content[:100])
            fragment = fragment.replace("__LT__", "<b>").replace("__GT__", "</b>")
            item['content'] = fragment
            item['title'] = doc.get("title")
            item['uri'] = doc.get("uri")
            item['source'] = doc.get("source")
            item['filetype'] = doc.get("filetype")
            item['score'] = hits.score(i)
            if DEBUG:
                item['explanation'] = self.searcher.explain(query, hits.id(i)).toHtml()
            result['hits'].append(item)

        whole_duration = time() - start
        
        if self.database_getter:
            self.database_getter().put("statistics", "search_time", search_duration)
            self.database_getter().put("statistics", "search_whole_time", whole_duration)

        result['search_time'] = search_duration
        result['search_whole_time'] = whole_duration

        if DEBUG:
            result['query_debug'] = query.toString()

        self.logger.info("search of '%s' took %f seconds (total %f seconds including the snippet and result generation) and got %d results"
            % (q, search_duration, whole_duration, hits.length()))

        return result
